########## Problem 7.1


### 1.
# Import the data:
voting <- read.csv("voting-demo.csv")

# Inspect the frequency distribution of the variable Choice:
t1 <- table(voting$Choice)
t2 <- transform(t1, cumulative=cumsum(Freq), relative=prop.table(Freq))
t2

# Non-parametric bootstrap for the proportion of people who vote for the SPD:
set.seed(42781)
n <- nrow(voting)  # Sample size.
K <- 10000  # Number of bootstrap replicates.
boots <- rep(NA, times=K)
for (k in 1:K) {
  resamp <- sample(voting$Choice, size=n, replace=TRUE)
  # Compute the proportion of people who vote for the SPD:
  boots[k] <- sum(resamp=="SPD")/n
}
hist(boots, breaks=30, freq=FALSE)


### 2.

# Extract the ages of voters and non-voters:
Age.vote <- voting$Age[voting$Vote=="Did vote"]
Age.novote <- voting$Age[voting$Vote=="Did not vote"]
mean(Age.vote, na.rm=TRUE)
mean(Age.novote, na.rm=TRUE)

# Calculate the size of each sample:
n.vote <- length(Age.vote)
n.novote <- length(Age.novote)

# The bootstrapping procedure:
set.seed(42781)
K <- 10000
boots <- rep(NA, times=K)
for (k in 1:K) {
  resamp.vote <- sample(Age.vote, size=n.vote, replace=TRUE)
  resamp.novote <- sample(Age.novote, size=n.novote, replace=TRUE)
  mean.vote <- mean(resamp.vote, na.rm=TRUE)
  mean.novote <- mean(resamp.novote, na.rm=TRUE)
  boots[k] <- mean.vote - mean.novote
}
hist(boots, breaks=30, freq=FALSE)

# Proportion of cases that the mean age of voters is greater than the mean age of non-voters:
sum(boots > 0)/K


### 3.

# Extract the variable Age and remove missing values:
voting.Age <- voting$Age
voting.Age <- voting.Age[-which(is.na(voting.Age))]

# The online bootstrapping procedure:
set.seed(42781)
K <- 10000

# We use starting values of 0 for the means, counts, and sums of squares:
online.means <- rep(0, times=K)
online.counts <- rep(0, times=K)
online.sumsofsquares <- rep(0, times=K)

# Function for online update of mean:
update.mean <- function(current, n, x) {
  return(current+(x-current)/n)
}

# Function for online update of the sum of squares:
update.sumofsquares <- function(current, xbar, n, d, x) {
  return(current+d*(x-xbar))
}

for (i in 1:length(voting.Age)) {
  # Sample:
  update <- sample(c(TRUE, FALSE), size=K, replace=TRUE)
  # Update:
  d <- voting.Age[i] - online.means
  online.counts[update] <- online.counts[update] + 1
  online.means[update] <- update.mean(current=online.means[update], n=online.counts[update], x=voting.Age[i])
  online.sumsofsquares[update] <- update.sumofsquares(current=online.sumsofsquares[update], xbar=online.means[update], n=online.counts[update], d=d[update], x=voting.Age[i])
}

# Use the online sums of squares and counts to compute the sample variances:
sample.variances <- online.sumsofsquares/(online.counts-1)
summary(sample.variances)
hist(sample.variances, breaks=50, freq=FALSE)




########## Problem 7.2


### 1.
# Import the data:
high_school <- read.csv("high-school.csv")

# Remove cases that do not receive an allowance:
high_school <- high_school[high_school$ALLOWANCE>0, ]

# Conduct the hypothesis test:
t.test(high_school$ALLOWANCE, mu=10)


### 2.
diff <- mean(high_school$ALLOWANCE) - 10
n <- length(high_school$ALLOWANCE)
s2 <- var(high_school$ALLOWANCE)
SE <- sqrt(s2/n)  # Estimated standard error of the mean.
CI.lower <- diff - SE*qnorm(1-0.05)
CI.upper <- diff + SE*qnorm(1-0.05)
-2 < CI.lower & CI.upper < 2


### 3.
qqnorm(high_school$ALLOWANCE)
qqline(high_school$ALLOWANCE)


### 4.
high_school$logALLOWANCE <- log(high_school$ALLOWANCE)
logALLOWANCE_boys <- high_school$logALLOWANCE[high_school$GENDER=="Boy"]
logALLOWANCE_girls <- high_school$logALLOWANCE[high_school$GENDER=="Girl"]
var.test(logALLOWANCE_boys, logALLOWANCE_girls)
t.test(logALLOWANCE_boys, logALLOWANCE_girls, var.equal=FALSE)


### 5.
boxplot(high_school$logALLOWANCE)
q1 <- quantile(high_school$logALLOWANCE, 0.25)
q3 <- quantile(high_school$logALLOWANCE, 0.75)
iqr <- q3 - q1
lower <- q1 - 1.5*iqr
upper <- q3 + 1.5*iqr
outlier <- high_school$logALLOWANCE < lower | high_school$logALLOWANCE > upper
sum(outlier)


### 6.
library(outliers)
grubbs.test(high_school$logALLOWANCE)




########## Problem 7.3


### 1.
# Import the data:
high_school <- read.csv("high-school.csv")

# Remove children that do not earn money:
high_school <- high_school[high_school$WORK>0, ]

# Examine normality of log transformed WORK for boys:
boys <- high_school[high_school$GENDER=="Boy", ]
boys$logWORK <- log(boys$WORK)
qqnorm(boys$logWORK)
qqline(boys$logWORK)


### 2.
library(outliers)
girls <- high_school[high_school$GENDER=="Girl", ]
girls$logWORK <- log(girls$WORK)
grubbs.test(girls$logWORK)


### 3.
t.test(boys$logWORK, girls$logWORK)


### 4.
diff <- mean(boys$logWORK) - mean(girls$logWORK)
n_boys <- length(boys$logWORK)
n_girls <- length(girls$logWORK)
s2_boys <- var(boys$logWORK)
s2_girls <- var(girls$logWORK)
SE <- sqrt(s2_boys/n_boys + s2_girls/n_girls)  # Estimated standard error of the difference between the two means.
CI.lower <- diff - SE*qnorm(1-0.05)
CI.upper <- diff + SE*qnorm(1-0.05)
-0.1 < CI.lower & CI.upper < 0.1


### 5.
breakfast <- high_school[high_school$BREAKFAST=="Yes", ]
no_breakfast <- high_school[high_school$BREAKFAST=="No", ]
t.test(breakfast$WORK, no_breakfast$WORK , alternative="greater")
