########## Problem 8.3


### 1.
# Import data:
houses <- read.csv("houses.csv")
# Create lotsize squared variable:
houses$lotsize2 <- houses$lotsize^2
# Calculate sample size:
n <- nrow(houses)

# Model 1:
M1 <- lm(price ~ lotsize, data=houses)  # Fit model.
design.M1 <- model.matrix(price ~ lotsize, data=houses)  # Check design matrix.
coef(M1)  # Extract estimated coefficients.
yhat.M1.in <- predict(M1)  # Calculate predicted values.
MSE.M1.in <- sum((houses$price - yhat.M1.in)^2) / n  # Calculate MSE.

# Model 2:
M2 <- lm(price ~ lotsize + lotsize2, data=houses)
design.M2 <- model.matrix(price ~ lotsize + lotsize2, data=houses)
coef(M2)
yhat.M2.in <- predict(M2)
MSE.M2.in <- sum((houses$price - yhat.M2.in)^2) / n

# Model 3:
M3 <- lm(price ~ lotsize + lotsize2 + bedrooms, data=houses)
design.M3 <- model.matrix(price ~ lotsize + lotsize2 + bedrooms, data=houses)
coef(M3)
yhat.M3.in <- predict(M3)
MSE.M3.in <- sum((houses$price - yhat.M3.in)^2) / n

# Model 4:
M4 <- lm(price ~ lotsize * bedrooms + lotsize2, data=houses)
design.M4 <- model.matrix(price ~ lotsize * bedrooms + lotsize2, data=houses)
coef(M4)
yhat.M4.in <- predict(M4)
MSE.M4.in <- sum((houses$price - yhat.M4.in)^2) / n

# Compare the within-sample MSE's:
c(MSE.M1.in, MSE.M2.in, MSE.M3.in, MSE.M4.in)

# Plot model 1:
plot(houses$lotsize, houses$price, xlab="Lot size", ylab="Price")
abline(M1)

# Plot model 2:
plot(houses$lotsize, houses$price, xlab="Lot size", ylab="Price")
b <- coef(M2)
curve(b[1]+b[2]*x+b[3]*x^2, from=min(houses$lotsize), to=max(houses$lotsize), add=TRUE)

# Plot model 3:
# Function for predicting the price based on lotsize (x) and bedrooms (y):
predict.price <- function(x, y) {
  new.data <- data.frame(lotsize=x, lotsize2=x^2, bedrooms=y)
  predict(M3, newdata=new.data)
}
# Generate sequences of lot sizes and bedrooms:
x <- seq(from=min(houses$lotsize), to=max(houses$lotsize), length.out=30)
y <- seq(from=min(houses$bedrooms), to=max(houses$bedrooms), length.out=30)
# Predict the price for all values in x and y (see ?outer for help):
z <- outer(x, y, FUN=predict.price)
# Create perspective plot:
p <- persp(x, y, z, xlab="Lot size", ylab="Bedrooms", zlab="Price",
           col="grey", ticktype="detailed", cex.axis=0.75, theta=-45, phi=20)
# Add points:
observed <- trans3d(x=houses$lotsize, y=houses$bedrooms, z=houses$price, pmat=p)
predicted <- trans3d(x=houses$lotsize, y=houses$bedrooms, z=fitted(M3), pmat=p)
points(observed, pch=20)
segments(x0=observed$x, y0=observed$y, x1=predicted$x, y1=predicted$y)


### 2.
AIC(M1, M2, M3, M4)
BIC(M1, M2, M3, M4)


### 3.
# Randomly shuffle the data:
set.seed(2241)
houses.shuffled <- houses[sample(1:n), ]
# Create 10 equally sized folds:
folds <- cut(1:n, breaks=10, labels=FALSE)

# Create function for cross validating different models:
cross.validate <- function(model.formula) {
  # Create vector for storing out-of-sample MSE's:
  MSE.out <- rep(NA, 10)
  # Perform 10-fold cross validation:
  for (i in 1:10) {
    # Segment data by fold using the which() function:
    testIndices <- which(folds==i)
    testData <- houses.shuffled[testIndices, ]
    trainData <- houses.shuffled[-testIndices, ]
    # Fit model on training data:
    M <- lm(model.formula, data=trainData)
    # Use model to predict house prices on test data:
    yhat.out <- predict(M, newdata=testData)
    # Compute MSE on test data:
    MSE.out[i] <- sum((testData$price - yhat.out)^2) / nrow(testData)
  }
  # Finally, compute the mean MSE over the cross validation sets:
  return(mean(MSE.out))
}

# Cross validate the four models:
MSE.M1.out <- cross.validate(formula(price ~ lotsize))
MSE.M2.out <- cross.validate(formula(price ~ lotsize + lotsize2))
MSE.M3.out <- cross.validate(formula(price ~ lotsize + lotsize2 + bedrooms))
MSE.M4.out <- cross.validate(formula(price ~ lotsize * bedrooms + lotsize2))

# Compare the out-of-sample MSE's:
c(MSE.M1.out, MSE.M2.out, MSE.M3.out, MSE.M4.out)


### 4.
library(MASS)
# Randomly shuffle the data:
set.seed(2241)
houses.shuffled <- houses[sample(1:n), ]
# Create 10 equally sized folds:
folds <- cut(1:n, breaks=10, labels=FALSE)

# Create function for cross validating different values of lambda:
cross.validate <- function(lambda) {
  # Create vector for storing out-of-sample MSE's:
  MSE.out <- rep(NA, 10)
  # Perform 10-fold cross validation:
  for (i in 1:10) {
    # Segment data by fold using the which() function:
    testIndices <- which(folds==i)
    testData <- houses.shuffled[testIndices, ]
    trainData <- houses.shuffled[-testIndices, ]
    # Fit model on training data:
    M <- lm.ridge(price ~ lotsize * bedrooms + lotsize2, data=trainData, lambda=lambda)
    # Create design matrix for test data:
    X <- model.matrix(price ~ lotsize * bedrooms + lotsize2, data=testData)
    # Extract estimated model coefficients:
    b <- coef(M)
    # Use estimated coefficients to predict house prices on test data:
    yhat.out <- X %*% b  # "%*%" performs matrix multiplication.
    # Compute MSE on test data:
    MSE.out[i] <- sum((testData$price - yhat.out)^2) / nrow(testData)
  }
  # Finally, compute the mean MSE over the cross validation sets:
  return(mean(MSE.out))
}

# Create sequence of lambdas between 0 and 5:
lambdas <- seq(from=0, to=5, by=0.1)
# Create vector for storing MSE's for different lambdas:
MSE.cv <- rep(NA, length(lambdas))
# Perform cross validation for different lambdas:
for (j in 1:length(lambdas)) {MSE.cv[j] <- cross.validate(lambdas[j])}
# Determine which value of lambda has minimal out-of-sample MSE:
lambda.min <- lambdas[which.min(MSE.cv)]
lambda.min

# Plot results:
plot(lambdas, MSE.cv)
abline(v=lambda.min)
