########## Problem 9.1


### 1.
set.seed(78)
n <- 50
mu <- 10
sigma2 <- 2
y <- rnorm(n, mean=mu, sd=sqrt(sigma2))


### 2.
post.mean.var <- function(y, sigma2, mu0, tau02) {
  n <- length(y)
  ybar <- mean(y)
  tau12 <- (1/tau02 + n/sigma2)^-1
  mu1 <- (mu0/tau02 + n*ybar/sigma2) * tau12
  results <- c(mu1, tau12)
  names(results) <- c("mu1", "tau12")
  return(results)
}

mu0 <- 12
tau02 <- 0.1
post <- post.mean.var(y, sigma2, mu0, tau02)
post


### 3.
ybar <- mean(y)
curve(dnorm(x, mean=mu0, sd=sqrt(tau02)),
      from=min(ybar, mu0)-5*sqrt(post[2]), to=max(ybar, mu0)+5*sqrt(post[2]),
      ylim=c(0, dnorm(post[1], mean=post[1], sd=sqrt(post[2]))),
      xlab=expression(mu), ylab="Density / Likelihood", cex.lab=1.2)


### 4.
curve(dnorm(x, mean=post[1], sd=sqrt(post[2])), add=TRUE, col="red")
legend("topright", legend=c("Prior distribution", "Posterior distribution"),
       lty=1, col=c("black", "red"), bg="white")
abline(v=ybar, col="grey")


### 6.
relative.likelihood <- function(y, mu, sigma2) {  # mu can be a vector.
  ybar <- mean(y)
  max.logL <- sum(dnorm(y, mean=ybar, sd=sqrt(sigma2), log=TRUE))  # Compute log likelihood at ybar.
  logL <- rep(NA, length(mu))
  for (i in 1:length(mu)) {
    logL[i] <- sum(dnorm(y, mean=mu[i], sd=sqrt(sigma2), log=TRUE))  # Compute log likelihood for values in mu.
  }
  relative.logL <- logL - max.logL  # Compute relative log likelihood.
  relative.L <- exp(relative.logL)  # Compute relative likelihood.
  return(relative.L)
}

curve(relative.likelihood(y, x, sigma2), add=TRUE, col="grey")
legend("topright", legend=c("Prior distribution", "Posterior distribution",
       "Relative likelihood"), lty=1, col=c("black", "red", "grey"),
       bg="white")




########## Problem 9.2


### 1.
simData <- function(n, theta) {
  return(sum(rbinom(n, size=1, prob=theta)))
}


### 2.
updateBelief <- function(params=c(1, 1), s, n) {
  params[1] <- params[1] + s  # Update alpha.
  params[2] <- params[2] + (n-s)  # Update beta.
  return(params)
}


### 3.
computeLoss <- function(d, theta) {
  return((d - theta)^2)
}


### 4.
expectedValueBeta <- function(params) {
  return(params[1] / sum(params))
}


### 5.
params <- c(1, 1)  # Specify uniform prior with alpha = beta = 1.
theta <- 0.2  # True theta is 0.2.
n <- 100  # Number of observations.
m <- 1000  # Number of simulation runs.
EvVector <- rep(NA, times=m)  # Results vector for the expected value of the posterior.

for (i in 1:m) {
  # Simulate data:
  s <- simData(n, theta)
  # Compute expected value and store:
  posterior.params <- updateBelief(params, s, n)
  EvVector[i] <- expectedValueBeta(posterior.params)
}


### 6.
meanEV <- mean(EvVector)
curve(computeLoss(x, theta), from=0, to=1, xlab="d", ylab="Loss")
abline(v=meanEV, col="red")
meanEV




########## Problem 9.3


### 1.
# Generate data and fit Bayesian regression model:
library(MCMCpack)
set.seed(13412348)
n <- 100
x <- runif(n, min=-10, max=10)
y <- rnorm(n, mean=2+0.5*x, sd=1)
bayes <- MCMCregress(y ~ x)

# Create posterior density plot of beta_1:
plot(density(bayes[, 2]), xlim=c(0.4, 0.6), xlab=expression(beta[1]),
     main=expression(paste("Posterior distibution of ", beta[1])), cex.lab=1.2)


### 2.
library(MASS)
z <- kde2d(bayes[, 1], bayes[, 2])
contour(z, xlab=expression(beta[0]), ylab=expression(beta[1]), cex.lab=1.2)
cor(bayes[, 1], bayes[, 2])


### 3.
# Draw from posterior predictive distribution of Y:
x <- 3
beta0 <- bayes[, 1]
beta1 <- bayes[, 2]
sigma2 <- bayes[, 3]
S <- nrow(bayes)
y <- rnorm(S, mean=beta0+beta1*x, sd=sqrt(sigma2))

# Plot the posterior predictive distribution:
plot(density(y), main="Posterior predictive distribution of Y", xlab="y")

# Compute point prediction for Y:
mean(y)

# Quantify uncertainty using standard deviation and 95% interval:
sd(y)
quantile(y, probs=c(0.025, 0.975))
